@@ -4,8 +4,6 @@
  * (C) 1994, 1995 Ulrich Callmeier
  */
 
-#define IGNORE_UNENC 1
-
 #include <sys/time.h>
 #include <sys/wait.h>
 #include "netacct.h"
@@ -18,18 +16,20 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <utmp.h>
+#include <pwd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <netinet/in.h>
+#include <netinet/ip_icmp.h>
 #include <netinet/protocols.h>
 #include <linux/if.h>
 #include <linux/if_ether.h>
 #include <linux/ip.h>
 #include <linux/tcp.h>
 
-char *rcs_revision_capture_c = "$Revision: 1.3 $";
+char *rcs_revision_capture_c = "$Revision: 1.9 $";
 
 void handle_ip(unsigned char buf[], char *devname, char *user);
 
@@ -351,6 +351,8 @@ void packet_loop()
   
   struct promisc_device *p;
 
+  struct headerdat *h;
+
   dynamicstyle = (dev2line != NULL) ? 1 : ((cfg->dynamicip != NULL) ? 2 : 0);
 
   buf = &buff[20];
@@ -369,32 +371,36 @@ void packet_loop()
       do_user = 0;
       
       p = cfg -> notdev;
-    
+      
       while(p!=NULL)
 	{
-		if (!strcmp(p -> name, saddr.sa_data)) {
-			packets->notdev++;
-			break;
-		}
-		p = p -> next;
+	  if (!strcmp(p -> name, saddr.sa_data)) {
+	    packets->notdev++;
+	    break;
+	  }
+	  p = p -> next;
 	}
-	if (p)
-		continue;
-
-      if((strncmp(saddr.sa_data,"eth",3)==0) || strncmp(saddr.sa_data,"lo",2)==0)
-	/* THIS NEEDS FIXING !! LIST IS INCOMPLETE */
+      if (p)
+	continue;
+      
+      h = cfg -> headers;
+      
+      hardheader = -1;
+      while (h)
 	{
-	  hardheader = 14;
-	  type = (buf[12] * 256 + buf[13]);
-		
-	  if(type != ETH_P_IP)
+	  if(strncmp(saddr.sa_data,h->name,h->l) == 0)
 	    {
-	      /* ETH_P_ARP, ETH_P_RARP, ETH_P_IPX, etc. */
-	      packets->ignored++;
-	      continue;
+	      hardheader = h -> offset;
+	      if (h -> type != 0)
+		type = buf[h -> type] * 256 + buf[h -> type + 1];
+	      else
+		type = ETH_P_IP;
+	      break;
 	    }
+	  h = h -> next;
 	}
-      else
+
+      if (hardheader == -1)
 	{
 	  /* ASSUMES: interface - line just with ppp and slip etc. */
 	  if(dynamicstyle == 1) do_user = 1;
@@ -406,6 +412,15 @@ void packet_loop()
 	  continue; /* ignore ppp/slip */
 #endif
 	}
+      else
+	{
+	  if(type != ETH_P_IP)
+	    {
+	      /* ETH_P_ARP, ETH_P_RARP, ETH_P_IPX, etc. */
+	      packets->ignored++;
+	      continue;
+	    }
+	}
 
       tmp_iphdr = (void *) (buf+hardheader);
       if((tmp_iphdr->saddr & cfg->ignoremask) == (tmp_iphdr->daddr & cfg->ignoremask))
@@ -432,66 +447,68 @@ void packet_loop()
 	    }
 	  packets->ip++;
 	  user = NULL;
-	 
+	  
 	  switch(dynamicstyle)
-	      {
-	      case 1:
-		if(do_user) user = check_user_dev2line(saddr.sa_data);
-		break;
-	      case 2:
-		dynamicaddr = otheraddr = 0;
-		if(onnet(tmp_iphdr->saddr, &cfg->dynamicnet)) 
-		  {
-		    dynamicaddr = tmp_iphdr->saddr;
-		    if (onnet(tmp_iphdr->daddr, &cfg->dynamicnet)) otheraddr = tmp_iphdr->daddr;
-		    DEBUG(DBG_ANNOYING, sprintf(dbg, "source %s is on dynamicnet\n", intoa(dynamicaddr)));
-
-		  }
-		else if (onnet(tmp_iphdr->daddr, &cfg->dynamicnet))
-		  {
-		    dynamicaddr = tmp_iphdr->daddr;
-		    if(onnet(tmp_iphdr->saddr, &cfg->dynamicnet)) otheraddr = tmp_iphdr->saddr;
-		    DEBUG(DBG_ANNOYING, sprintf(dbg, "destination %s is on dynamicnet\n", intoa(dynamicaddr)));
-		  }
-		
-		if(dynamicaddr != 0)
-		  {
-		    if(onnetlist(dynamicaddr, cfg->excludenamelookup))
-		      {
-			DEBUG(DBG_ANNOYING, sprintf(dbg, "BUT: %s is excluded from name lookup\n", intoa(dynamicaddr)));
-
-			dynamicaddr = otheraddr;
-			otheraddr = 0;
-			if(onnetlist(dynamicaddr, cfg->excludenamelookup))
-			  {
-			    DEBUG(DBG_ANNOYING, sprintf(dbg, "prev. bug: %s is excluded from name lookup, too\n", intoa(dynamicaddr)));
-			    dynamicaddr = 0;
-			  }
-		      }
-		    if(dynamicaddr != 0)
-		      {
-			user = check_user_dynamicip(dynamicaddr); 
-			if((user == NULL) && (otheraddr != 0))
-			  {
-			    user = check_user_dynamicip(dynamicaddr); 
-			  }
-		      }
-		  }
-		break;
-	      }
-	    handle_ip(buf+hardheader, saddr.sa_data, user);
-	  }
-      }
+	    {
+	    case 1:
+	      if(do_user) user = check_user_dev2line(saddr.sa_data);
+	      break;
+	    case 2:
+	      dynamicaddr = otheraddr = 0;
+	      if(onnet(tmp_iphdr->saddr, &cfg->dynamicnet)) 
+		{
+		  dynamicaddr = tmp_iphdr->saddr;
+		  if (onnet(tmp_iphdr->daddr, &cfg->dynamicnet)) otheraddr = tmp_iphdr->daddr;
+		  DEBUG(DBG_ANNOYING, sprintf(dbg, "source %s is on dynamicnet\n", intoa(dynamicaddr)));
+		  
+		}
+	      else if (onnet(tmp_iphdr->daddr, &cfg->dynamicnet))
+		{
+		  dynamicaddr = tmp_iphdr->daddr;
+		  if(onnet(tmp_iphdr->saddr, &cfg->dynamicnet)) otheraddr = tmp_iphdr->saddr;
+		  DEBUG(DBG_ANNOYING, sprintf(dbg, "destination %s is on dynamicnet\n", intoa(dynamicaddr)));
+		}
+	      
+	      if(dynamicaddr != 0)
+		{
+		  if(onnetlist(dynamicaddr, cfg->excludenamelookup))
+		    {
+		      DEBUG(DBG_ANNOYING, sprintf(dbg, "BUT: %s is excluded from name lookup\n", intoa(dynamicaddr)));
+		      
+		      dynamicaddr = otheraddr;
+		      otheraddr = 0;
+		      if(onnetlist(dynamicaddr, cfg->excludenamelookup))
+			{
+			  DEBUG(DBG_ANNOYING, sprintf(dbg, "prev. bug: %s is excluded from name lookup, too\n", intoa(dynamicaddr)));
+			  dynamicaddr = 0;
+			}
+		    }
+		  if(dynamicaddr != 0)
+		    {
+		      user = check_user_dynamicip(dynamicaddr); 
+		      if((user == NULL) && (otheraddr != 0))
+			{
+			  user = check_user_dynamicip(otheraddr); 
+			}
+		    }
+		}
+	      break;
+	    }
+	  handle_ip(buf+hardheader, saddr.sa_data, user);
+	}
+    }
 }
 
 void handle_ip(unsigned char buf[], char *devname, char *user)
 {
     static struct iphdr *tmp_iphdr;
     static struct tcphdr *tmp_tcphdr;
+    static struct icmphdr *tmp_icmphdr;
     unsigned short srcport, dstport;
     tmp_iphdr = (void *) &buf[0];
     tmp_tcphdr = (void *) &buf[tmp_iphdr->ihl*4];
     /* relevant headers of udphdr and tcphdr are identical */
+    tmp_icmphdr = (void *) &buf[tmp_iphdr->ihl*4];
 
     switch(tmp_iphdr->protocol)
 	{
@@ -511,7 +528,8 @@ void handle_ip(unsigned char buf[], char *devname, char *user)
 	    break;
 	case IP_ICMP:
 	    packets->ip_icmp++;
-	    srcport = dstport = 0;
+	    srcport = tmp_icmphdr->type; 
+	    dstport = tmp_icmphdr->code;
 	    break;
 	default:
 	    packets->ip_other++;
@@ -521,3 +539,52 @@ void handle_ip(unsigned char buf[], char *devname, char *user)
     
     register_packet(tmp_iphdr->saddr,tmp_iphdr->daddr,tmp_iphdr->protocol, srcport, dstport, ntohs(tmp_iphdr->tot_len), devname, user);
 }
+
+#ifdef TCP_USER_INFO
+
+/* Originally by Vlad Seriakov (vlad@torn.ktts.kharkov.ua) */
+/* Adapted by Ulrich Callmeier */
+
+int get_tcp_info(struct ipdata *ip, uid_t *uid)
+{
+  FILE *procinfo;
+  char buffer[1024];
+  int num, local_port, rem_port, d, state, tmout;
+  long localaddr, remaddr;
+  unsigned long rxq, txq, time_len, retr, inode;
+  int timer_run;
+  
+  if ((procinfo = fopen(PATH_PROCNET_TCP, "r")) == NULL)
+    {
+      syslog(LOG_ERR, "error opening %s: %s", PATH_PROCNET_TCP,strerror(errno));
+      DEBUG(DBG_ERR, sprintf(dbg,"error opening %s: %s", PATH_PROCNET_TCP,strerror(errno)));
+      return(-1);
+    }
+  
+  fgets(buffer, sizeof(buffer), procinfo); /* skip header line */
+  
+  while (!feof(procinfo))
+    {
+      if(fgets(buffer, sizeof(buffer), procinfo)!=NULL)
+	{
+	  num = sscanf(buffer,
+		       "%d: %lX:%X %lX:%X %X %lX:%lX %X:%lX %lX %d %d %ld\n",
+		       &d, &localaddr, &local_port,
+		       &remaddr, &rem_port, &state,
+		       &txq, &rxq, &timer_run, &time_len, &retr, (int*) uid, &tmout, &inode);
+	  if (num < 11) continue;		/* 13 ? */
+	  if(localaddr == ip->src && remaddr == ip->dst &&
+	     local_port == ip->srcport && rem_port == ip->dstport)
+	    {
+	      fclose(procinfo);
+	      return 1;
+	    }
+	}
+    }
+  
+  fclose(procinfo);
+  
+  return(0);
+}
+
+#endif
