@@ -11,14 +11,14 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <malloc.h>
-#include <strings.h>
+#include <string.h>
 #include <signal.h>
 #include <fcntl.h>
 #include <utmp.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 
-char *rcs_revision_process_c = "$Revision: 1.2be $";
+char *rcs_revision_process_c = "$Revision: 1.7 $";
 
 volatile int running;
 
@@ -98,69 +98,98 @@ void do_acct()
 
 #define DISABLED(n) (cfg->disabled & BITMASK(n))
 
-/* simplified version, performance ??? */
 void register_packet(unsigned long int src,unsigned long int dst, unsigned char proto, unsigned short srcport, unsigned short dstport, int size, char *devname, char *user)
 {
-    if(lck==0)
+  uid_t uid;
+
+  if(lck==0)
+    {
+      struct ipdata *p;
+      DEBUG(DBG_ANNOYING, sprintf(dbg, "lck = 1\n"));
+      lck = 1;
+      p = plist;
+      list_lookups++;
+      while(p)
 	{
-	    struct ipdata *p;
-	    lck = 1;
-	    p = plist;
-	    list_lookups++;
-	    while(p)
-		{
-		    list_compares++;
-		    if( (DISABLED(DIS_PROTO) || (p->proto == proto)) && ( DISABLED(DIS_SRC) || (p->src == src)) && (DISABLED(DIS_DST) || (p->dst == dst)) &&  (DISABLED(DIS_SRCPORT) || (p->srcport == srcport)) && (DISABLED(DIS_DSTPORT) || (p->dstport == dstport)) && (strcmp(p->devname, devname)==0))
-			{
-			    p->bytes +=size;
-			    p->when = now;
-                            p->count++;
-			    lck = 0;
-			    return;
-			}
-		    p = p->next;
-		}
-	    p = malloc(sizeof(struct ipdata));
-	    if(p == NULL)
-		{
-		    packets -> dropped++;
-		    return;
-		}
-	    plistsize++;
-	    p -> src = src;
-	    p -> dst = dst;
-	    p -> proto = proto;
-	    p -> srcport = srcport;
-	    p -> dstport = dstport;
-	    p -> bytes = size;
-	    p -> count = 1;
-	    p -> devname = strdup(devname);
-            if (user)
-	        p -> user = strdup(user);
-	    else
-	        p -> user = NULL;
-	    p -> next = plist;
-	    p -> when = now;
-	    
-	    plist = p;
-	    lck = 0;
+	  list_compares++;
+	  if( (DISABLED(DIS_PROTO) || (p->proto == proto)) && ( DISABLED(DIS_SRC) || (p->src == src)) && (DISABLED(DIS_DST) || (p->dst == dst)) &&  (DISABLED(DIS_SRCPORT) || (p->srcport == srcport)) && (DISABLED(DIS_DSTPORT) || (p->dstport == dstport)) && (strcmp(p->devname, devname)==0))
+	    {
+	      p->bytes +=size;
+	      p->when = now;
+	      p->count++;
+	      lck = 0;
+	      DEBUG(DBG_ANNOYING, sprintf(dbg, "lck = 0\n"));
+	      return;
+	    }
+	  p = p->next;
 	}
-    else
+      p = malloc(sizeof(struct ipdata));
+      if(p == NULL)
 	{
-	    packets->dropped++;
+	  packets -> dropped++;
+	  lck = 0;
+	  DEBUG(DBG_ERR, sprintf(dbg, "out of memory\n"));
+	  DEBUG(DBG_ANNOYING, sprintf(dbg, "lck = 0\n"));
+	  return;
 	}
+      plistsize++;
+      p -> src = src;
+      p -> dst = dst;
+      p -> proto = proto;
+      p -> srcport = srcport;
+      p -> dstport = dstport;
+      p -> bytes = size;
+      p -> count = 1;
+      p -> devname = strdup(devname);
+      if (user)
+	p -> user = strdup(user);
+      else
+	p -> user = NULL;
+      p -> next = plist;
+      p -> when = now;
+      
+#ifdef TCP_USER_INFO
+      /* NEW !!! get user for tcp only */
+      if((proto == IP_TCP) && (p->user == 0) && (get_tcp_info(p, &uid)==1))
+	{
+	  char tmp[16];
+	  sprintf(tmp, "<%d>", uid);
+	  p -> user = strdup(tmp);
+	}
+#endif
+      
+      plist = p;
+      lck = 0;
+      DEBUG(DBG_ANNOYING, sprintf(dbg, "lck = 0\n"));
+    }
+  else
+    {
+      packets->dropped++;
+    }
 }
 
 int do_write_list(FILE *f, struct ipdata *list)
 {
     struct ipdata *p;
+#ifdef HUMAN_READABLE_TIME
+    struct tm *tm1;
+    time_t tmm;
+#endif
 
     p = list;
 
     while(p)
 	{
-	    if(fprintf(f, "%lu",time(0))<0) return 1;
-	    
+#ifdef HUMAN_READABLE_TIME
+	  /* by Vlad Seriakov */
+	  time(&tmm);
+	  tm1 = localtime(&tmm); 
+	  if(fprintf(f, "%02d/%02d/%02d %02d:%02d:%02d",tm1->tm_year,tm1->tm_mon,tm1->tm_mday,
+		     tm1->tm_hour,tm1->tm_min,tm1->tm_sec)<0) return 1;
+#else
+	  if(fprintf(f, "%lu",time(0))<0) return 1;
+#endif	
+    
 	    if(!DISABLED(DIS_PROTO)) if(fprintf(f, "\t%d",p->proto)<0) return 1;
 
 	    if(!DISABLED(DIS_SRC)) if(fprintf(f, "\t%s",intoa(p->src))<0) return 1;
@@ -177,7 +206,7 @@ int do_write_list(FILE *f, struct ipdata *list)
 
 	    if(!DISABLED(DIS_DEV)) if(fprintf(f, "\t%s",p->devname)<0) return 1;
 
-	    if(!DISABLED(DIS_USER)) if(fprintf(f, "\t%s\n",p->user?p->user:"unknown")<0) return 1;
+	    if(!DISABLED(DIS_USER)) if(fprintf(f, "\t%s",p->user?p->user:"unknown")<0) return 1;
 
 	    if(fprintf(f, "\n")<0) return 1;
    
@@ -531,6 +560,7 @@ void write_log(int force)
 					list_lookups, list_compares, 
 					((float) list_compares / (float) list_lookups)));
 	}
+      DEBUG(DBG_STATE, sprintf(dbg, "lck = 1\n"));
       lck = 1; /* can't update the list now */
       
       DEBUG(DBG_MISC, sprintf(dbg, "Total of %ld entries\n", plistsize));
@@ -585,7 +615,9 @@ void write_log(int force)
 	}
       
       writing = 1; /* no further writing 'til this is finished */
+      
       lck = 0;
+      DEBUG(DBG_STATE, sprintf(dbg, "lck = 0\n"));
       
       DEBUG(DBG_MISC, sprintf(dbg, "Split into %ld [hold] and %ld [write] = %ld [total] entries\n", plistsize, olistsize, plistsize + olistsize));
       
